@@ -6,23 +6,46 @@ import pycurl
 from types import *
 import ThreadPool
 import time
-import os
+import os,os.path
 import glob
 import BTDownloader
+import HTTPDownloader
 from utils import format_size
 
 import utils
+superglobal=utils.SuperGlobal()
 
 import gnome
-#this isn't going to work right now, it only notifies when download is complete
+import gnomevfs
+
+from penguintv import DOWNLOAD_ERROR, DOWNLOAD_PROGRESS, DOWNLOAD_WARNING, DOWNLOAD_QUEUED
+
+FINISHED=0
+FINISHED_AND_PLAY=1
+STOPPED=2
+FAILURE=-1
+
+
+#Downloader API:
+#constructor takes:  media, params, resume, queue, progress_callback, finished_callback
+#  media:  the media dic
+#  params: optional params, like for btdownloader
+#  resume: are we supposed to resume?
+#  queue:  are we supposed queue for playback when download is finished?  this variable is just passed around
+#  progress_callback:  function to call for progress update.
+#          arg of this is: (media, progress as 0 < x < 1, and text formatted message of progress)
+#          the callback will return 1 if we should cancel download
+#  finished_callback:  function to call when finished.
+#          args is: (media, status, message)
+#              where status is the enum above
 
 class MediaManager:
 	def __init__(self, progress_callback=None, finished_callback=None):
 		self.index=0
-		self.pool = ThreadPool.ThreadPool(5)
+		self.pool = ThreadPool.ThreadPool(5,"MediaManager")
 		self.db = ptvDB.ptvDB()
 		self.time_appendix=0
-		self.bt_settings = {}
+		self.bt_settings = {'min_port':6881, 'max_port':6999, 'ul_limit':0}
 		self.id_time=0
 		self.quitting = False
 		if finished_callback:
@@ -85,36 +108,58 @@ class MediaManager:
 		"""queues a download"""
 		media = self.db.get_media(media_id)
 		media['downloader_index']=self.index
-		media['download_status']=1			
+		media['download_status']=1
+		media.setdefault('size',0)			
 		if media['file'] is None:
 			filename = os.path.basename(media['url'])
 			filen, ext = os.path.splitext(filename)
 			ext = ext.split('?')[0] #grrr lugradio...
-			media['file']=self.media_dir+"/"+utils.get_dated_dir()+"/"+filen+"-"+self.get_id()+ext
-		else:
+			media['file']=self.media_dir+"/"+utils.get_dated_dir()+"/"+filen+ext
 			dated_dir = os.path.split(os.path.split(media['file'])[0])[1]
 			try: #make sure
 				os.stat(self.media_dir+"/"+dated_dir)
 			except:
 				os.mkdir(self.media_dir+"/"+dated_dir)
+			if self.db.media_exists(media['file']): #if the filename is in the db, rename
+				media['file']=self.media_dir+"/"+utils.get_dated_dir()+"/"+filen+"-"+self.get_id()+ext
+			else:
+				try:
+					os.stat(media['file'])  #if this raises exception, the file doesn't exist and we're ok
+					media['file']=self.media_dir+"/"+utils.get_dated_dir()+"/"+filen+"-"+self.get_id()+ext #if not, get new name
+				except:
+					pass #we're ok
+			
 			if resume==False:
 				self.db.delete_media(media_id)
 			#else:
 			#	print "resuming using existing filename: "+str(media['file'])
-		extension = media['url'][media['url'].rfind(".")+1:]
-		if extension.upper()=="TORRENT":
+		extension = os.path.splitext(media['url'])[1]
+		
+		if media['mimetype'] == 'application/x-bittorrent' or extension.upper()==".TORRENT":
 			params = [
 				'--minport', str(self.bt_settings['min_port']),
 				'--maxport', str(self.bt_settings['max_port']),
 				'--max_upload_rate', str(self.bt_settings['ul_limit'])]
 				
-			downloader = BTDownloader.BTDownloader(media, params, self.pool_progress_callback,self.pool_finished_callback, queue)
+			downloader = BTDownloader.BTDownloader(media, self.media_dir, params,True, queue, self.pool_progress_callback,self.pool_finished_callback)
 			self.pool.queueTask(downloader.download)
 			pass
-		else:
-			downloader = self._downloader(media, self.pool_progress_callback, resume)
-			self.pool.queueTask(downloader.download,queue,self.pool_finished_callback)
+		else: #http regular download
+			ext = os.path.splitext(media['file'])[1]
+			if len(ext)>5 or len(ext)==0:
+				#I think this isn't really the right extension.   See fucking ask a ninja: http://feeds.feedburner.com/AskANinja
+				try:
+					import mimetypes
+					real_ext = mimetypes.guess_extension(media['mimetype'])
+					if real_ext is not None:
+						media['file']=media['file']+real_ext
+				except:
+					print "ERROR couldn't guess mimetype, leaving filename alone"
+			downloader = HTTPDownloader.HTTPDownloader(media, self.media_dir, None, resume, queue, self.pool_progress_callback, self.pool_finished_callback)
+			self.pool.queueTask(downloader.download)
+			
 		self.db.set_media_download_status(media['media_id'],1)
+		superglobal.download_status[media['media_id']] = (DOWNLOAD_QUEUED,0,media['size'])
 		#self.db.set_media_viewed(media['media_id'],False)
 		self.db.set_media_filename(media['media_id'],media['file'])
 		self.index=self.index+1
@@ -124,89 +169,37 @@ class MediaManager:
 			return 1
 		self.pool_progress_callback(data)
 		
-	class _downloader:
-		"""Need a little internal class to keep track of callbacks from urllib.urlretrieve"""
-		def __init__(self, media, progress_callback, resume=False):
-			self.media = media
-			self.progress_callback = progress_callback
-			self.resume = resume
-			
-		def download(self, queue=False):
-			db = ptvDB.ptvDB()
-			try:
-				os.makedirs(os.path.dirname(self.media['file']))
-			except OSError:
-				pass
-			try:
-				if self.resume:
-					try:
-						fp = open(self.media['file'], "ab")
-					except:
-						fp = open(self.media['file'], "wb")
-				else:
-					fp = open(self.media['file'], "wb")
-				curl = pycurl.Curl()
-				curl.setopt(pycurl.URL, str(self.media['url'])) #cause it's unicode or some shit which is not a string or some junk
-				curl.setopt(pycurl.FOLLOWLOCATION, 1)
-				curl.setopt(pycurl.MAXREDIRS, 5)
-				curl.setopt(pycurl.CONNECTTIMEOUT, 30)
-				#curl.setopt(pycurl.TIMEOUT, 300)
-				curl.setopt(pycurl.NOSIGNAL, 1)
-				curl.setopt(pycurl.WRITEDATA, fp)
-				curl.setopt(pycurl.PROGRESSFUNCTION, self.wrap_progress_callback)
-				curl.setopt(pycurl.NOPROGRESS, 0)
-				curl.setopt(pycurl.USERAGENT,'PenguinTV '+utils.VERSION)
-				if self.resume:
-					#print os.stat(self.media['file'])
-					cursize = os.stat(self.media['file'])[6]
-					#print "current size "+str(cursize)
-					curl.setopt(pycurl.RESUME_FROM_LARGE, cursize)
-				curl.perform()
-				curl.close()
-				fp.close()
-			except Exception, data:
-				if data[0]==33: #if server doesn't support resuming, retry
-					print data
-					print "retrying without resume"
-					self.resume=False
-					return self.download(queue)
-				print "some downloading error "+str(data)
-				self.media['errormsg']=data
-				return(self.media,0,data)
-			if queue:
-				return (self.media, 2,"finished downloading "+self.media['file'])
-			return (self.media, 1,"finished downloading "+self.media['file'])
-			
-		def wrap_progress_callback(self, dl_total, dl_now, ul_total, ul_now):
-			try:
-				progress = int((dl_now*100.0)/dl_total)
-			except:
-				progress = 0
-			result = self.progress_callback((self.media,progress,"Downloaded "+str(progress)+"% of "+format_size(self.media['size'])))
-			return result
-			
 	def _basic_finished_callback(self, data):
-		filename = data[0]['file']
-		info = data[1:]
-		print filename+" "+str(info)
+		#filename = data[0]['file']
+		#info = data[1:]
+		#print filename+" "+str(info)
+		print data
+		self.db.set_media_download_status(data[0]['media_id'],ptvDB.D_DOWNLOADED)	
 			
 	def _basic_progress_callback(self, data):
-		media, blocks, blocksize, totalsize=data
-		percent = (blocks*blocksize*100) / totalsize
-		if percent>100:
-			percent = 100
-		if percent%10 == 0:
-			print media['file']+" "+str(percent)+"%"
-	
-	def cancel_download(self, media_id):
-		#find the thread with the right media_id
-		#let it DIE
-		pass
-	
-	def pause_download(self, media_id):
-		#can't do right now
-		pass
+		#media, blocks, blocksize, totalsize=data
+		#percent = (blocks*blocksize*100) / totalsize
+		#if percent>100:
+	#		percent = 100
+		#if percent%10 == 0:
+		#	print media['file']+" "+str(percent)+"%"
+		print os.path.split(data[0]['file'])[1]+" "+data[2]
+			
+	def get_download_count(self):
+		try:
+			return self.pool.getTaskCount()
+		except:
+			return 0
 		
+	def pause_all_downloads(self):
+		#print "downloads paused"
+		try:
+			self.pool.joinAll(False,True) #don't wait for tasks, but let the threads die naturally
+			if self.quitting == False:
+				self.pool.setThreadCount(5)
+		except:
+			pass
+			
 	def get_disk_usage(self):
 		size = 0
 		try:
