@@ -36,6 +36,8 @@ class Player:
 			
 		if type(files) != list:
 			files = [files]
+			
+		players={}
 
 		for f in files:
 			if os.path.isdir(f):
@@ -43,16 +45,36 @@ class Player:
 					for filen in filelist:
 						next = os.path.join(f, filen)
 						if os.path.isfile(next) and utils.is_known_media(filen):
-							cmdline=cmdline+" "+next+" "
 							head,filename = os.path.split(next)
 							dated_dir = os.path.split(head)[1]
 							playlist.write(dated_dir+"/"+filename+"\n")
+							
+							player = utils.get_play_command_for(filen)
+							if players.has_key(player):
+								players[player].append(filen)
+							else:
+								players[player]=[filen]
+							
 			elif os.path.isfile(f):
-				cmdline=cmdline+" "+f+" "
 				head,filename = os.path.split(f)
 				dated_dir = os.path.split(head)[1]
 				playlist.write(dated_dir+"/"+filename+"\n")
+				player = utils.get_play_command_for(f)
+				if players.has_key(player):
+					players[player].append(f)
+				else:
+					players[player]=[f]
 		playlist.close()
-		#print cmdline
-		subProcess.subProcess(cmdline+" &")
+		for player in players.keys():
+			cmdline=player+" "
+			for filename in players[player]:
+				cmdline+=filename+" "
+			cmdline+="&"
+			#print "running: "+str(cmdline)
+			subProcess.subProcess(cmdline)
 			
+class NoDir(Exception):
+	def __init__(self,durr):
+		self.durr = durr
+	def __str__(self):
+		return "no such directory: "+self.durr
